# Copyright 2013 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
#
# Refer to the README and COPYING files for full details of the license
#

import os
import sys
from distutils.sysconfig import get_python_lib

from testrunner import VdsmTestCase as TestCaseBase
from monkeypatch import MonkeyPatch


def path_without_vdsm_ext_mod():
    sub_dirs = ['client', 'vdsm', 'vdsm_api', '/usr/share/vdsm']
    exclude_dirs = [os.path.abspath(
                    os.path.join(os.path.dirname(__file__), '..', sub_dir))
                    for sub_dir in sub_dirs]
    return [path for path in sys.path
            if os.path.abspath(path) not in exclude_dirs]


def get_mods(path):
    return [(file.strip('.py')).strip('.pyc') for file in
            os.listdir(os.path.abspath(path))
            if (file.endswith(".py") or file.endswith(".pyc"))]


class CrossImportsTestCaseShould(TestCaseBase):

    @MonkeyPatch(sys, 'path', path_without_vdsm_ext_mod())
    def test_fail_when_importing_ext_mod(self):
        pkg_name = 'vdsm'
        src_dir = '../lib/%s' % pkg_name
        if os.path.exists(src_dir):
            mods = get_mods(src_dir)
        else:
            mods = get_mods(os.path.join(get_python_lib(), pkg_name))

        __import__(pkg_name, fromlist=mods)
